import {
  IconStrikethrough,
  IconTextBold,
  IconTextItalic,
  IconTextUnderline,
} from "@/assets/icons";
import { useEditor } from "@/modules/editor";
import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { LinkOutlined } from "@ant-design/icons-vue";

import { css } from "@linaria/core";
import { Button, Divider, InputNumber, Select } from "ant-design-vue";
import { defineComponent } from "vue";
import { any } from "vue-types";

import ToolbarsUI from "../../formItems/ToolbarsUI";
import { AlignComp, LineHeightComp, TextColor } from "./ToolbarComp";

export const TextToolbar = defineComponent({
  props: {
    component: any<DesignComp>().isRequired,
  },
  setup(props) {
    const { actions } = useEditor();
    const toolbarColumns = [
      {
        dataIndex: "fontFamily",
        component: Select,
        props: {
          class: "w-180px",
          defaultValue: "",
          options: [
            { label: "默认字体", value: "" },
            { label: "宋体", value: "宋体,Songti,STSong,serif" },
            { label: "黑体", value: "黑体,Heiti,STHeiti,sans-serif" },
            { label: "仿宋", value: "仿宋,FangSong,STFangsong,serif" },
            { label: "楷体", value: "楷体,KaiTi,STKaiti,sans-serif" },
          ],
        },
      },
      {
        label: "字号",
        dataIndex: "fontSize",
        component: InputNumber,
        props: {
          class: "!w-66px",
          defaultValue: 12,
          min: 12,
          max: 99,
        },
      },
      {
        label: "字体颜色",
        dataIndex: "color",
        component: TextColor,
      },
      {
        label: "加粗",
        dataIndex: "bold",
        component: () => <Button icon={<IconTextBold />} type="text"></Button>,
        itemProps: {
          class: "!mx-2px",
        },
      },
      {
        label: "斜体",
        dataIndex: "italic",
        component: () => (
          <Button icon={<IconTextItalic />} type="text"></Button>
        ),
        itemProps: {
          class: "!mx-2px",
        },
      },
      {
        label: "下划线",
        dataIndex: "underline",
        component: () => (
          <Button icon={<IconTextUnderline />} type="text"></Button>
        ),
        itemProps: {
          class: "!mx-2px",
        },
      },
      {
        label: "删除线",
        dataIndex: "strikethrough",
        component: () => (
          <Button icon={<IconStrikethrough />} type="text"></Button>
        ),
        itemProps: {
          class: "!mx-2px",
        },
      },
      {
        component: () => (
          <Divider
            type="vertical"
            style={{ fontSize: "26px", borderColor: "#1f1f1f" }}
          />
        ),
      },
      {
        label: "对齐方式",
        dataIndex: "align",
        component: AlignComp,
        itemProps: {
          class: "!mx-2px",
        },
      },
      {
        label: "行高",
        dataIndex: "lineHeight",
        component: LineHeightComp,
        itemProps: {
          class: "!mx-2px",
        },
      },
      {
        component: () => (
          <Divider
            type="vertical"
            style={{ fontSize: "26px", borderColor: "#1f1f1f" }}
          />
        ),
      },
      {
        label: "链接",
        dataIndex: "align",
        component: () => <Button icon={<LinkOutlined />} type="text"></Button>,
        itemProps: {
          class: "!mx-2px",
        },
      },
    ];
    function changeVal(e: { dataIndex: string; value: any }) {
      actions.updateCompData(props.component, e.dataIndex, e.value);
    }
    return () => {
      const { component } = props;
      return (
        <div class={[TextToolbarStyle, "flex items-center"]}>
          <ToolbarsUI
            data={component}
            columns={toolbarColumns}
            onChange={changeVal}
          />
        </div>
      );
    };
  },
});
const TextToolbarStyle = css``;
