import {
  IconTextCenter,
  IconTextJustify,
  IconTextLeft,
  IconTextRight,
  IconLineHeight,
} from "@/assets/icons";
import { css } from "@linaria/core";
import { Button, Dropdown, Menu, Tooltip } from "ant-design-vue";

import { defineComponent, onMounted } from "vue";
import { string } from "vue-types";
import Pickr from "@simonwep/pickr";
import "@simonwep/pickr/dist/themes/nano.min.css";
export const TextColor = defineComponent({
  props: {
    value: string().def("#666666"),
  },
  emits: ["change"],
  setup(props, { emit }) {
    let picker: any = null;

    function initPicker() {
      picker = Pickr.create({
        el: ".color_picker",
        theme: "nano",
        default: props.value,
        i18n: {
          "btn:save": "确定",
        },
        // swatches: defaultColor,
        components: {
          preview: true,
          opacity: false,
          hue: true,
          interaction: {
            hex: false,
            rgba: false,
            input: true,
            save: true,
          },
        },
      });

      picker.on("change", (color: any) => {
        const hexa = color.toHEXA().toString();
        // emit("change", hexa);
      });
      picker.on("save", (color: any) => {
        picker.hide();
        const hexa = color.toHEXA().toString();
        emit("change", hexa);
      });
    }
    onMounted(() => {
      initPicker();
    });

    return () => (
      <div
        class={ColorPicker}
        onClick={() => {
          picker?.show();
        }}
      >
        <div class={"color_picker"}></div>
      </div>
    );
  },
});
const ColorPicker = css`
  position: relative;
  width: 32px;
  height: 32px;
  border-radius: 2px;
  cursor: pointer;
  .pickr {
    width: 100%;
    height: 100%;

    .pcr-button {
      width: 100%;
      height: 100%;
      border: 1px solid transparent;
    }
    border-radius: 2px;
    input {
      &:focus,
      &.pcr-active {
        border-color: @inf-primary-color;
        box-shadow: 0 0 0 2px rgba(232, 139, 0, 0.2);
      }
    }
    button {
      &:focus,
      &.pcr-active {
        border-color: @inf-primary-color;
        box-shadow: 0 0 0 2px rgba(232, 139, 0, 0.2);
      }
    }
  }
  .pcr-app {
    input {
      &:focus,
      &.pcr-active {
        border-color: @inf-primary-color;
        box-shadow: 0 0 0 2px rgba(232, 139, 0, 0.2);
      }
    }
    button {
      &:focus,
      &.pcr-active {
        border-color: @inf-primary-color;
        box-shadow: 0 0 0 2px rgba(232, 139, 0, 0.2);
      }
    }
  }
`;
export const AlignComp = defineComponent({
  props: {
    value: string<"left" | "right" | "center" | "justify">().def("left"),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const aligns = [
      {
        label: "左对齐",
        key: "left",
        icon: <IconTextLeft />,
      },
      {
        label: "居中对齐",
        key: "center",
        icon: <IconTextCenter />,
      },
      {
        label: "右对齐",
        key: "right",
        icon: <IconTextRight />,
      },
      {
        label: "两端对齐",
        key: "justify",
        icon: <IconTextJustify />,
      },
    ];

    const overlay = () => {
      return (
        <Menu>
          {aligns.map((e: any) => {
            return (
              <Menu.Item
                class={props.value == e.key ? currStyle : null}
                onClick={() => {
                  emit("change", e.key);
                }}
              >
                <Tooltip placement="right" title={e.label}>
                  {e.icon}
                </Tooltip>
              </Menu.Item>
            );
          })}
        </Menu>
      );
    };
    const currIcon = (key: string) => {
      const item = aligns.find((e) => {
        return e.key == key;
      });
      return item ? item.icon : null;
    };
    return () => (
      <Dropdown overlay={overlay()} trigger={"click"} placement="bottom">
        <Button type="text" icon={currIcon(props.value)}></Button>
      </Dropdown>
    );
  },
});

export const LineHeightComp = defineComponent({
  props: {
    value: string().def("1.5"),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const options = [1, 1.5, 2, 2.5, 3];

    const overlay = () => {
      return (
        <Menu>
          {options.map((e: any) => {
            return (
              <Menu.Item
                class={props.value == e ? currStyle : null}
                onClick={() => {
                  emit("change", e);
                }}
              >
                {e}
              </Menu.Item>
            );
          })}
        </Menu>
      );
    };

    return () => (
      <Dropdown overlay={overlay()} trigger={"click"} placement="bottom">
        <Button type="text" icon={<IconLineHeight />}></Button>
      </Dropdown>
    );
  },
});
const currStyle = css`
  color: @inf-primary-color;
`;
