import { useEditor } from "@/modules/editor";
import { inject, onMounted, provide, ref } from "vue";

export function useCompRef(compId: string) {
  const compRef = ref();
  const { store, helper } = useEditor();

  const parentId = compId !== "root" ? (inject("compParentId") as string) : "";
  provide("compParentId", compId);
  store.setCompPid(compId, parentId);

  onMounted(() => {
    const comp = helper.findComp(compId);
    if (comp) {
      Object.defineProperties(comp, {
        $el: {
          value: compRef.value,
          configurable: true,
        },
      });
      compRef.value.compId= compId;
      compRef.value.compKey= comp.compKey;
    }
  });
  return compRef;
}

export function useCompEditLayerRef(compId: string) {

  const editLayerRef = ref<HTMLElement>();
  const { helper, store } = useEditor();

  onMounted(() => {
      if ( !editLayerRef.value ) return;
      const comp = helper.findComp(compId);
      if (comp) {
          Object.defineProperties(comp, {
          $editor: {
              value: editLayerRef.value,
              configurable: true,
          },
          });
          const dom = editLayerRef.value
          helper.setCompEditLayer(compId, ()=>{
              const {left, top, width, height} = dom.getBoundingClientRect();
               return {
                  left,top, width,height
               }
          })
      }
  });
  return editLayerRef;

}