import Empty from "@/components/Empty";
import { useEditor } from "@/modules/editor";
import { useResource } from "@/modules/resource";
import { Image, Loadmore } from "@queenjs/ui";
import { defineUI } from "queenjs";
import { Container, Draggable } from "vue-dndrop";

export default defineUI({
  setup() {
    const editor = useEditor();
    const source = useResource();
    const ctrl = source.controls.sysTplListCtrl;
    ctrl.loadPage(1);

    return () => {
      const dataSource = ctrl.state.list;

      return (
        <div>
          <Container
            class="space-y-10px"
            behaviour="copy"
            group-name="canvas"
            animation-duration={0}
            get-child-payload={(index: number) => {
              return {
                type: "tpl",
                data: dataSource[index],
              };
            }}
          >
            {dataSource.map((item: any) => {
              return (
                <Draggable key={item._id}>
                  <div
                    class="text-center leading-50px bg-dark-50 rounded draggable-item"
                    key={item._id}
                    title={item.title}
                    style={{ aspectRatio: 1 }}
                    onClick={() => editor.actions.clickTplToDesign(item)}
                  >
                    <Image
                      class="w-full rounded pointer-events-none"
                      src={item.thumbnail}
                      size={240}
                    />
                  </div>
                </Draggable>
              );
            })}
          </Container>
          {dataSource.length == 0 ? (
            <Empty class="pt-150px" />
          ) : (
            <Loadmore
              class="mt-20px"
              loading={ctrl.state.loading}
              canLoad={ctrl.state.canLoadNext}
              onChange={ctrl.loadNextPage}
            />
          )}
        </div>
      );
    };
  },
});
